;; Larry Mulcahy 1988
;; types

(provide 'type)
(require 'string)
(require 'sequence)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; *xlisp-types*
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar *xlisp-types*
  '(subr fsubr cons symbol fixnum flonum string object stream
    vector closure char ustream))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; XLISP-style-type-name 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun XLISP-style-type-name (type)
  (case type
    (integer 'fixnum)
    (flonum 'flonum)
    (list 'cons)
    (t type)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; typep 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun typep (object type)
  (if (eq type t)
      t
    (let ((type-name (XLISP-style-type-name (if (atom type) type (car type))))
         )
      (if (range-type-spec-p type)
          (and (equal (type-of object) type-name)
               (<= (second type) object (third type)))
          (equal (type-of object) type-name)))))

; Return a string giving a pretty description of a type for printing.
; ASK uses this to come up with articulate error messages.
; Messages should fit 
; "Input must be ..."

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; type-pretty-print-string 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun type-pretty-print-string (type-spec)
  (cond
   ;; *** One word type spec like "integer" or "string" ***
   ((atom type-spec)
    (let ((str (remove-hyphens-and-downcase (symbol-name type-spec))))
      (word-plus-indefinite-article str)))
   ;; *** Range check type-spec like "(number 1 3)"
   ((range-type-spec-p type-spec)
    (let ((str (remove-hyphens-and-downcase (symbol-name (first type-spec)))))
      (concatenate 'string (word-plus-indefinite-article str)
                           " in the range "
                           (primitive-number-to-string (second type-spec))
                           " ... "
                           (primitive-number-to-string (third type-spec)))))
   ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; range-type-spec-p 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun range-type-spec-p (type-spec)
  (and (not (atom type-spec))
       (equal (length type-spec) 3)
       (numberp (second type-spec))
       (numberp (third type-spec))))

